<?php
/**
 * Appender - Adds copyrights to the copied text.
 * Exclusively on Envato Market: https://1.envato.market/appender
 *
 * @encoding        UTF-8
 * @version         1.0.1
 * @copyright       Copyright (C) 2018 - 2019 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         dmitry@merkulov.design
 **/

namespace Merkulove\Appender;

use Merkulove\Appender;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Class used to implement plugin settings.
 *
 * @since 1.0.0
 * @author Alexandr Khmelnytsky (info@alexander.khmelnitskiy.ua)
 **/
final class Settings {

	/**
	 * Appender Plugin settings.
	 *
	 * @var array()
	 * @since 1.0.0
	 **/
	public $options = [];

	/**
	 * The one true Settings.
	 *
	 * @var Settings
	 * @since 1.0.0
	 **/
	private static $instance;

	/**
	 * Sets up a new Settings instance.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	private function __construct() {

		/** Get plugin settings. */
		$this->get_options();

	}

	/**
	 * Render Tabs Headers.
	 *
	 * @param string $current - Selected tab key.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function render_tabs( $current = 'general' ) {

		/** Tabs array. */
		$tabs          = [];
		$tabs['general'] = [
			'icon' => 'tune',
			'name' => esc_html__( 'General', 'appender' )
		];

        $tabs['assignments'] = [
            'icon' => 'flag',
            'name' => esc_html__( 'Assignments', 'appender' )
        ];

		$tabs['css'] = [
			'icon' => 'code',
			'name' => esc_html__( 'Custom CSS', 'appender' )
		];

		/** Activation tab enable only if plugin have Envato ID. */
		$plugin_id = EnvatoItem::get_instance()->get_id();
		if ( (int)$plugin_id > 0 ) {
			$tabs['activation'] = [
				'icon' => 'vpn_key',
				'name' => esc_html__( 'Activation', 'appender' )
			];
		}

		$tabs['status'] = [
			'icon' => 'info',
			'name' => esc_html__( 'Status', 'appender' )
		];

		$tabs['uninstall'] = [
			'icon' => 'delete_sweep',
			'name' => esc_html__( 'Uninstall', 'appender' )
		];

		/** Render Tabs. */
		?>
        <aside class="mdc-drawer">
            <div class="mdc-drawer__content">
                <nav class="mdc-list">

                    <div class="mdc-drawer__header mdc-plugin-fixed">
                        <!--suppress HtmlUnknownAnchorTarget -->
                        <a class="mdc-list-item mdp-plugin-title" href="#wpwrap">
                            <i class="mdc-list-item__graphic" aria-hidden="true">
                                <img src="<?php echo esc_attr( Appender::$url . 'images/logo-color.svg' ); ?>" alt="<?php echo esc_html__( 'Appender', 'appender' ) ?>">
                            </i>
                            <span class="mdc-list-item__text">
                                <?php echo esc_html__( 'Appender', 'appender' ) ?>
                                <sup><?php echo esc_html__( 'ver.', 'appender' ) . esc_html( Appender::$version ); ?></sup>
                            </span>
                        </a>
                        <button type="submit" name="submit" id="submit"
                                class="mdc-button mdc-button--dense mdc-button--raised">
                            <span class="mdc-button__label"><?php echo esc_html__( 'Save changes', 'appender' ) ?></span>
                        </button>
                    </div>

                    <hr class="mdc-plugin-menu">
                    <hr class="mdc-list-divider">
                    <h6 class="mdc-list-group__subheader"><?php echo esc_html__( 'Plugin settings', 'appender' ) ?></h6>

					<?php

					// Plugin settings tabs
					foreach ( $tabs as $tab => $value ) {
						$class = ( $tab == $current ) ? ' mdc-list-item--activated' : '';
						echo "<a class='mdc-list-item " . $class . "' href='?page=mdp_appender_settings&tab=" . $tab . "'><i class='material-icons mdc-list-item__graphic' aria-hidden='true'>" . $value['icon'] . "</i><span class='mdc-list-item__text'>" . $value['name'] . "</span></a>";
					}

					/** Helpful links. */
					$this->support_link();

					/** Activation Status. */
					PluginActivation::get_instance()->display_status();

					?>
                </nav>
            </div>
        </aside>
		<?php
	}

	/**
	 * Displays useful links for an activated and non-activated plugin.
	 *
	 * @since 1.0.0
     *
     * @return void
	 **/
	public function support_link() { ?>

        <hr class="mdc-list-divider">
        <h6 class="mdc-list-group__subheader"><?php echo esc_html__( 'Helpful links', 'appender' ) ?></h6>

        <a class="mdc-list-item" href="https://docs.merkulov.design/tag/appender/" target="_blank">
            <i class="material-icons mdc-list-item__graphic" aria-hidden="true"><?php echo esc_html__( 'collections_bookmark' ) ?></i>
            <span class="mdc-list-item__text"><?php echo esc_html__( 'Documentation', 'appender' ) ?></span>
        </a>

		<?php if ( PluginActivation::get_instance()->is_activated() ) : /** Activated. */ ?>
            <a class="mdc-list-item" href="https://1.envato.market/appender-support" target="_blank">
                <i class="material-icons mdc-list-item__graphic" aria-hidden="true">mail</i>
                <span class="mdc-list-item__text"><?php echo esc_html__( 'Get help', 'appender' ) ?></span>
            </a>
            <a class="mdc-list-item" href="https://1.envato.market/cc-downloads" target="_blank">
                <i class="material-icons mdc-list-item__graphic" aria-hidden="true">thumb_up</i>
                <span class="mdc-list-item__text"><?php echo esc_html__( 'Rate this plugin', 'appender' ) ?></span>
            </a>
		<?php endif; ?>

        <a class="mdc-list-item" href="https://1.envato.market/cc-merkulove" target="_blank">
            <i class="material-icons mdc-list-item__graphic" aria-hidden="true"><?php echo esc_html__( 'store' ) ?></i>
            <span class="mdc-list-item__text"><?php echo esc_html__( 'More plugins', 'appender' ) ?></span>
        </a>
		<?php

	}

	/**
	 * Add plugin settings page.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function add_settings_page() {

		add_action( 'admin_menu', [ $this, 'add_admin_menu' ] );
		add_action( 'admin_init', [ $this, 'settings_init' ] );

	}

	/**
	 * Create General Tab.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
    public function general_tab() {

	    /** General Tab. */
	    $group_name = 'AppenderOptionsGroup';
	    $section_id = 'mdp_appender_settings_page_general_section';
	    $option_name = 'mdp_appender_settings';

	    /** Settings fields on general tab. */
	    $fields = [
		    'string_length'         => esc_html__( 'String length:', 'appender' ),
		    'add_before'            => esc_html__( 'Before copied:', 'appender' ),
		    'text_before'           => '',
		    'add_after'             => esc_html__( 'After copied:', 'appender' ),
		    'text_after'            => '',
		    'save'                  => esc_html__( 'Disable Save:', 'appender' ),
            'view_source'           => esc_html__( 'Disable View Source:', 'appender' ),
            'developer_tool'        => esc_html__( 'Disable Developer Tool:', 'appender' ),
            'safari_reader'         => esc_html__( 'Disable Safari Reader Mode:', 'appender' ),
            'js_required'           => esc_html__( 'JavaScript Required:', 'appender' ),
            'js_required_message'   => esc_html__( 'JavaScript Required Message:', 'appender' ),
	    ];

	    /** Create settings section. */
	    register_setting( $group_name, $option_name );
	    add_settings_section( $section_id, '', null, $group_name );

	    /** Call render_$key for each field. */
	    foreach ( $fields as $key => $label ) {

		    /** Skip if render method not exist. */
		    if ( ! is_callable( [SettingsFields::get_instance(), 'render_' . $key] ) ) { continue; }
		    add_settings_field( $key, $label, [SettingsFields::get_instance(), 'render_' . $key ], $group_name, $section_id );

	    }

    }

	/**
	 * Create Custom CSS Tab.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function custom_css_tab() {

		/** Custom CSS. */
		$group_name = 'AppenderCSSOptionsGroup';
		$section_id = 'mdp_appender_settings_page_css_section';

		/** Create settings section. */
		register_setting( $group_name, 'mdp_appender_css_settings' );
		add_settings_section( $section_id, '', null, $group_name );

    }

	/**
	 * Generate Settings Page.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function settings_init() {

		/** General Tab. */
	    $this->general_tab();

		/** Create Assignments Tab. */
		AssignmentsTab::get_instance()->add_settings();

		/** Create Custom CSS Tab. */
		$this->custom_css_tab();

		/** Activation Tab. */
		PluginActivation::get_instance()->add_settings();

		/** Create Status Tab. */
		StatusTab::get_instance()->add_settings();

		/** Create Uninstall Tab. */
		UninstallTab::get_instance()->add_settings();

	}

	/**
	 * Add admin menu for plugin settings.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function add_admin_menu() {

		add_menu_page(
			esc_html__( 'Appender Settings', 'appender' ),
			esc_html__( 'Appender', 'appender' ),
			'manage_options',
			'mdp_appender_settings',
			[ $this, 'options_page' ],
			'data:image/svg+xml;base64,' . base64_encode( file_get_contents( Appender::$path . 'images/logo-menu.svg' ) ),
			'58.6485'// Always change digits after "." for different plugins.
		);

	}

	/**
	 * Plugin Settings Page.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function options_page() {

		/** User rights check. */
		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		} ?>
        <!--suppress HtmlUnknownTarget -->
        <form action='options.php' method='post'>
            <div class="wrap">

				<?php
				$tab = 'general';
				if ( isset ( $_GET['tab'] ) ) { $tab = $_GET['tab']; }

				/** Render "Appender settings saved!" message. */
				$this->render_nags();

				/** Render Tabs Headers. */
				?><section class="mdp-aside"><?php $this->render_tabs( $tab ); ?></section><?php

				/** Render Tabs Body. */
				?><section class="mdp-tab-content mdp-tab-<?php echo esc_attr( $tab ) ?>"><?php

					/** General Tab. */
					if ( $tab == 'general' ) {
						echo '<h3>' . esc_html__( 'Appender Settings', 'appender' ) . '</h3>';
						settings_fields( 'AppenderOptionsGroup' );
						do_settings_sections( 'AppenderOptionsGroup' );

                    /** Assignments Tab. */
					} elseif ( $tab == 'assignments' ) {
						echo '<h3>' . esc_html__( 'Assignments Settings', 'appender' ) . '</h3>';
						settings_fields( 'AppenderAssignmentsOptionsGroup' );
						do_settings_sections( 'AppenderAssignmentsOptionsGroup' );
						AssignmentsTab::get_instance()->render_assignments();

                    /** Custom CSS Tab. */
					} elseif ( $tab == 'css' ) {
						echo '<h3>' . esc_html__( 'Custom CSS', 'appender' ) . '</h3>';
						settings_fields( 'AppenderCSSOptionsGroup' );
						do_settings_sections( 'AppenderCSSOptionsGroup' );
						SettingsFields::get_instance()->render_custom_css();

						/** Activation Tab. */
					} elseif ( $tab == 'activation' ) {
						settings_fields( 'AppenderActivationOptionsGroup' );
						do_settings_sections( 'AppenderActivationOptionsGroup' );
						PluginActivation::get_instance()->render_pid();

                    /** Status tab. */
					} elseif ( $tab == 'status' ) {
						echo '<h3>' . esc_html__( 'System Requirements', 'appender' ) . '</h3>';
						StatusTab::get_instance()->render_form();

					} /** Uninstall Tab. */
                    elseif ( $tab == 'uninstall' ) {
						echo '<h3>' . esc_html__( 'Uninstall Settings', 'appender' ) . '</h3>';
						UninstallTab::get_instance()->render_form();
					}

					?>
                </section>
            </div>
        </form>

		<?php
	}

	/**
	 * Render "Settings Saved" nags.
	 *
	 * @since 1.0.0
     *
     * @return void
	 **/
	public function render_nags() {

	    /** Did we try to save settings? */
		if ( ! isset( $_GET['settings-updated'] ) ) { return; }

		/** Are the settings saved successfully? */
		if ( $_GET['settings-updated'] === 'true' ) {

			/** Render "Settings Saved" message. */
			UI::get_instance()->render_snackbar( esc_html__( 'Settings saved!', 'appender' ) );
		}

		if ( ! isset( $_GET['tab'] ) ) { return; }

		if ( strcmp( $_GET['tab'], "activation" ) == 0 ) {

			if ( PluginActivation::get_instance()->is_activated() ) {

				/** Render "Activation success" message. */
				UI::get_instance()->render_snackbar( esc_html__( 'Plugin activated successfully.', 'appender' ), 'success', 5500 );

			} else {

				/** Render "Activation failed" message. */
				UI::get_instance()->render_snackbar( esc_html__( 'Invalid purchase code.', 'appender' ), 'error', 5500 );

			}

		}

	}

	/**
	 * Get plugin settings with default values.
	 *
	 * @since 1.0.0
	 * @access public
	 * @return void
	 **/
	public function get_options() {

		/** Options. */
		$options = get_option( 'mdp_appender_settings' );

		/** CSS tab Options. */
		$mdp_appender_css_settings = get_option( 'mdp_appender_css_settings' );

		/** Default values. */
		$defaults = [
			'string_length'         => 0,
			'add_before'            => isset( $options[ 'add_before' ] ) ? $options[ 'add_before' ] : 'off',
			'text_before'           => esc_html__( 'This text copied from site {SiteTitle} {SiteURL}', 'appender' ),
			'add_after'             => isset( $options[ 'add_after' ] ) ? $options[ 'add_after' ] : 'on',
			'text_after'            => esc_html__( 'Read more: {PageTitle} {PageURL}', 'appender' ),
			'save'                  => isset( $options[ 'save' ] ) ? $options[ 'save' ] : 'on',
			'view_source'           => isset( $options[ 'view_source' ] ) ? $options[ 'view_source' ] : 'on',
			'developer_tool'        => isset( $options[ 'developer_tool' ] ) ? $options[ 'developer_tool' ] : 'on',
			'safari_reader'         => isset( $options[ 'safari_reader' ] ) ? $options[ 'safari_reader' ] : 'on',
			'js_required'           => isset( $options[ 'js_required' ] ) ? $options[ 'js_required' ] : 'on',
			'js_required_message'   => '<h4>' . esc_html__( 'Please Enable JavaScript in your Browser to Visit this Site.', 'appender' ) . '</h4>',
			'custom_css'            => isset( $mdp_appender_css_settings['custom_css'] ) ? $mdp_appender_css_settings['custom_css'] : '', // Custom CSS.
		];

		$results = wp_parse_args( $options, $defaults );

		$this->options = $results;
	}

	/**
	 * Main Settings Instance.
	 *
	 * Insures that only one instance of Settings exists in memory at any one time.
	 *
	 * @static
	 * @return Settings
	 * @since 1.0.0
	 **/
	public static function get_instance() {
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Settings ) ) {
			self::$instance = new Settings;
		}

		return self::$instance;
	}

} // End Class Settings.
