<?php
/**
 * Appender - Adds copyrights to the copied text.
 * Exclusively on Envato Market: https://1.envato.market/appender
 *
 * @encoding        UTF-8
 * @version         1.0.1
 * @copyright       Copyright (C) 2018 - 2019 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         dmitry@merkulov.design
 **/

namespace Merkulove\Appender;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Class used to implement plugin SettingsFields.
 *
 * @since 1.0.0
 * @author Alexandr Khmelnytsky (info@alexander.khmelnitskiy.ua)
 **/
final class SettingsFields {

	/**
	 * The one true SettingsFields.
	 *
	 * @var SettingsFields
	 * @since 1.0.0
	 **/
	private static $instance;

	/** @noinspection PhpUnused */
	/**
	 * Render Text length field.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function render_string_length() {

		/** Render Text Length slider. */
	    UI::get_instance()->render_slider(
		    Settings::get_instance()->options['string_length'],
            0,
            500,
            1,
		    '',
		    esc_html__( 'Copied string length:', 'appender' ) . ' <strong>' . Settings::get_instance()->options['string_length'] . '</strong> ' . esc_html__( 'characters', 'appender' ),
		    [
			    'name' => 'mdp_appender_settings[string_length]',
			    'id' => 'mdp_appender_settings_string_length',
                'class' => 'mdc-slider-width'
		    ]

        );

	}

	/** @noinspection PhpUnused */
	/**
	 * Render Add Text Before field.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function render_add_before() {

		/** Render Add Text Before switcher. */
		UI::get_instance()->render_switches(
			Settings::get_instance()->options['add_before'],
			esc_html__('Add before the copied text', 'appender' ),
			'',
			[
				'name' => 'mdp_appender_settings[add_before]',
				'id' => 'mdp_appender_add_before'
			]
		);

	}

	/** @noinspection PhpUnused */
	/**
	 * Render Text Before field.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function render_text_before() {

		/** Render Text Before switcher. */
		UI::get_instance()->render_textarea(
			Settings::get_instance()->options['text_before'],
			'',
			esc_html__( 'This text will be added in the beginning of Copy & Pasted Text.', 'appender' ),
			[
				'name' => 'mdp_appender_settings[text_before]',
				'id' => 'mdp_appender_settings_text_before'
			]
		);

	}

	/** @noinspection PhpUnused */
	/**
	 * Render Add Text After field.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function render_add_after() {

		/** Render Add Text After switcher. */
		UI::get_instance()->render_switches(
			Settings::get_instance()->options['add_after'],
			esc_html__('Add after the copied text', 'appender' ),
			'',
			[
				'name' => 'mdp_appender_settings[add_after]',
				'id' => 'mdp_appender_add_after'
			]
		);

	}

	/** @noinspection PhpUnused */
	/**
	 * Render Text Before field.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function render_text_after() {

		/** Render Text After switcher. */
		UI::get_instance()->render_textarea(
			Settings::get_instance()->options['text_after'],
			'',
			esc_html__( 'This text will be added to the end of Copy & Pasted Text.', 'appender' ),
			[
				'name' => 'mdp_appender_settings[text_after]',
				'id' => 'mdp_appender_settings_text_after'
			]
		);

	}

	/** @noinspection PhpUnused */
	/**
	 * Render Disable Save field.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function render_save() {

		/** Render Disable Save switcher. */
		UI::get_instance()->render_switches(
			Settings::get_instance()->options['save'],
			esc_html__( 'Protect from being Saved by HotKeys', 'appender' ),
			esc_html__( 'Disable: ', 'appender' ) . '<b>Ctrl</b>' . ' + ' . '<b>S</b>' . esc_html__( ' (Windows and Linux), ', 'appender' ) . '<b>⌘</b>' . ' + ' . '<b>S</b>' . esc_html__( ' (macOS)', 'appender' ),
			[
				'name' => 'mdp_appender_settings[save]',
				'id' => 'mdp_appender_settings_save'
			]
        );

	}

	/** @noinspection PhpUnused */
	/**
	 * Render Disable View Source field.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function render_view_source() {

		/** Render Disable View Source switcher. */
		UI::get_instance()->render_switches(
			Settings::get_instance()->options['view_source'],
			esc_html__( 'Protect source code viewing by HotKeys', 'appender' ),
			esc_html__( 'Disable: ', 'appender' ) . '<b>Ctrl</b>' . ' + ' . '<b>U</b>' . esc_html__( ' (Windows and Linux), ', 'appender' ) . '<b>⌘</b>' . ' + ' . '<b>U</b>' . esc_html__( ' (macOS)', 'appender' ),
			[
				'name' => 'mdp_appender_settings[view_source]',
				'id' => 'mdp_appender_settings_view_source'
			]
		);

	}

	/** @noinspection PhpUnused */
	/**
	 * Render Disable Developer Tool field.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function render_developer_tool() {

		/** Render Disable Developer Tool switcher. */
		UI::get_instance()->render_switches(
			Settings::get_instance()->options['developer_tool'],
			esc_html__( 'Protect source code viewing by Developer Tools', 'appender' ),
			esc_html__( 'Disable: ', 'appender' ) . '<b>Ctrl</b>' . ' + ' . '<b>Shift</b>' . ' + ' . '<b>I</b>' . esc_html__( ' (Windows and Linux), ', 'appender' ) . '<b>⌘</b>' . ' + ' . '<b>⌥</b>' . ' + ' . '<b>I</b>' . esc_html__( ' (macOS)', 'appender' ),
			[
				'name' => 'mdp_appender_settings[developer_tool]',
				'id' => 'mdp_appender_settings_developer_tool'
			]
		);

	}

	/** @noinspection PhpUnused */
	/**
	 * Render Disable Safari Reader Mode field.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function render_safari_reader() {

		/** Render Disable Safari Reader Mode switcher. */
		UI::get_instance()->render_switches(
			Settings::get_instance()->options['safari_reader'],
			esc_html__( 'Protect from being copied in the Safari Reader mode', 'appender' ),
			esc_html__( 'Disable: ', 'appender' ) . '<b>⌘</b>' . ' + ' . '<b>Shift</b>' . ' + ' . '<b>R</b>' . esc_html__( ' Safari (macOS)', 'appender' ),
			[
				'name' => 'mdp_appender_settings[safari_reader]',
				'id' => 'mdp_appender_settings_safari_reader'
			]
		);

	}

	/** @noinspection PhpUnused */
	/**
	 * Render JavaScript Required field.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function render_js_required() {

		/** Render JavaScript Required switcher. */
		UI::get_instance()->render_switches(
			Settings::get_instance()->options['js_required'],
			esc_html__( 'Protect content if JavaScript is disabled', 'appender' ),
			__( 'Show JavaScript Required Message.', 'appender' ),
			[
				'name' => 'mdp_appender_settings[js_required]',
				'id' => 'mdp_appender_settings_js_required'
			]
		);

	}

	/** @noinspection PhpUnused */
	/**
	 * Render 'JavaScript Required Message' field.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function render_js_required_message() {

		/** @noinspection SpellCheckingInspection */
		wp_editor( Settings::get_instance()->options['js_required_message'], 'mdpappendersettingsjsrequiredmessage', [
			'textarea_rows' => 7,
			'textarea_name' => 'mdp_appender_settings[js_required_message]'
		] );

	}

	/**
	 * Render CSS field.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function render_custom_css() {
		?>
		<div>
            <label>
                <textarea
                    id="mdp_custom_css_fld"
                    name="mdp_appender_css_settings[custom_css]"
                    class="mdp_custom_css_fld"><?php echo esc_textarea( Settings::get_instance()->options['custom_css'] ); ?></textarea>
            </label>
			<p class="description"><?php esc_html_e( 'Add custom CSS here.', 'appender' ); ?></p>
		</div>
		<?php
	}

	/**
	 * Main SettingsFields Instance.
	 *
	 * Insures that only one instance of SettingsFields exists in memory at any one time.
	 *
	 * @static
	 * @return SettingsFields
	 * @since 1.0.0
	 **/
	public static function get_instance() {
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof SettingsFields ) ) {
			self::$instance = new SettingsFields;
		}

		return self::$instance;
	}

} // End Class SettingsFields.
